@@ -262,6 +262,7 @@ MsRunReader::MsRunReader(const MsRunReader &other)
   : mcsp_msRunId(other.mcsp_msRunId)
 {
   mpa_multiMapScanNumber = nullptr;
+  m_isMonoThread         = other.m_isMonoThread;
 }
 
 
@@ -278,6 +279,18 @@ MsRunReader::~MsRunReader()
     delete mpa_multiMapScanNumber;
 }
 
+void
+MsRunReader::setMonoThread(bool is_mono_thread)
+{
+  m_isMonoThread = is_mono_thread;
+}
+
+bool
+MsRunReader::isMonoThread() const
+{
+  return m_isMonoThread;
+}
+
 
 std::size_t
 MsRunReader::scanNumber2SpectrumIndex(std::size_t scan_number)
