@@ -169,8 +169,8 @@ class PMSPP_LIB_DECL MsRunReaderTicChromatogram
   public:
   MsRunReaderTicChromatogram();
   virtual ~MsRunReaderTicChromatogram();
-  virtual void
-  setQualifiedMassSpectrum(const QualifiedMassSpectrum &qualified_mass_spectrum) override;
+  virtual void setQualifiedMassSpectrum(
+    const QualifiedMassSpectrum &qualified_mass_spectrum) override;
   virtual bool needPeakList() const override;
 
   Trace getTicChromatogram() const;
@@ -278,6 +278,15 @@ class PMSPP_LIB_DECL MsRunReader
 
   virtual Trace getTicChromatogram();
 
+
+  /** @brief set only one is_mono_thread to true
+   *
+   * this avoid to use qtconcurrent
+   */
+  void setMonoThread(bool is_mono_thread);
+
+  bool isMonoThread() const;
+
   protected:
   MsRunIdCstSPtr mcsp_msRunId;
   MsRunReaderScanNumberMultiMap *mpa_multiMapScanNumber = nullptr;
@@ -289,6 +298,9 @@ class PMSPP_LIB_DECL MsRunReader
    * format
    */
   virtual bool accept(const QString &file_name) const = 0;
+
+  private:
+  bool m_isMonoThread = false;
 };
 
 
