@@ -32,7 +32,6 @@
 #include "../../exception/exceptionnotfound.h"
 #include "../../msrun/xiccoord/xiccoordtims.h"
 #include <QDebug>
-#include <QtConcurrent/QtConcurrent>
 
 using namespace pappso;
 
@@ -61,7 +60,6 @@ pappso::TimsMsRunReaderMs2::initialize()
                     "msp_timsData is null for MsRunId %1")
           .arg(mcsp_msRunId.get()->toString()));
     }
-    
 }
 
 void
@@ -219,6 +217,9 @@ TimsMsRunReaderMs2::readSpectrumCollectionByMsLevel(
   // the user.
   handler.spectrumListHasSize(spectrum_list_size);
 
+
+  msp_timsData.get()->setMonoThread(isMonoThread());
+
   msp_timsData.get()->ms2ReaderSpectrumCollectionByMsLevel(
     getMsRunId(), handler, ms_level);
   // Now let the loading handler know that the loading of the data has ended.
@@ -324,4 +325,3 @@ TimsMsRunReaderMs2::getTicChromatogram()
 
   return msp_timsData->getTicChromatogram();
 }
-
