@@ -383,27 +383,28 @@ std::vector<double>
 MzIntegrationParams::createArbitraryBins()
 {
 
-  // qDebug();
+  //qDebug();
 
   // Now starts the tricky stuff. Depending on how the binning has been
   // configured, we need to take diverse actions.
 
-  // qDebug() << "Bin size:" << mp_precision->toString();
+  //qDebug() << "Bin specification:" << mp_precision->toString();
 
   pappso::pappso_double min_mz = m_smallestMz;
   pappso::pappso_double max_mz = m_greatestMz;
 
-  // qDebug() << QString::asprintf("min_mz: %.6f\n", min_mz)
-  //<< QString::asprintf("max_mz: %.6f\n", max_mz);
+  //qDebug() << QString::asprintf("min_mz: %.6f\n", min_mz)
+           //<< QString::asprintf("max_mz: %.6f\n", max_mz);
 
   pappso::pappso_double binSize = mp_precision->delta(min_mz);
 
-  // qDebug() << QString::asprintf(
-  //"binSize is the precision delta for min_mz: %.6f\n", binSize);
+  //qDebug() << QString::asprintf(
+    //"binSize is the precision delta for min_mz: %.6f\n", binSize);
 
   // Only compute the decimal places if they were not configured already.
   if(m_decimalPlaces == -1)
     {
+      // qDebug() << "Now checking how many decimal places are needed.";
 
       // We want as many decimal places as there are 0s between the integral
       // part of the double and the first non-0 cipher.  For example, if
@@ -412,8 +413,8 @@ MzIntegrationParams::createArbitraryBins()
 
       m_decimalPlaces = pappso::Utils::zeroDecimalsInValue(binSize) + 1;
 
-      // qDebug() << "With the binSize m_decimalPlaces was computed to be:"
-      //<< m_decimalPlaces;
+      // qDebug() << "With binSize" << binSize
+      //<< " m_decimalPlaces was computed to be:" << m_decimalPlaces;
     }
 
   // Now that we have defined the value of m_decimalPlaces, let's use that
@@ -456,11 +457,15 @@ MzIntegrationParams::createArbitraryBins()
   while(previous_mz_bin <= last_mz)
     {
 
+      // qDebug() << "Now starting the bin creation loop.";
+
       // Calculate dynamically the precision delta according to the current mz
       // value.
 
-      double current_mz =
-        previous_mz_bin + mp_precision->delta(previous_mz_bin);
+      //double precision_delta = mp_precision->delta(previous_mz_bin);
+      // qDebug() << "precision_delta: " << precision_delta;
+
+      double current_mz = previous_mz_bin + mp_precision->delta(previous_mz_bin);
 
       // qDebug() << QString::asprintf(
       //"previous_mzBin: %.6f and current_mz: %.6f\n",
@@ -492,9 +497,9 @@ MzIntegrationParams::createArbitraryBins()
             ceil((current_mz * pow(10, m_decimalPlaces)) - 0.49) /
             pow(10, m_decimalPlaces);
 
-          qDebug().noquote()
-            << "Had to increment decimal places by one while creating the bins "
-               "in BinningType::ARBITRARY mode..";
+          // qDebug().noquote()
+          //<< "Had to increment decimal places by one while creating the bins "
+          //"in BinningType::ARBITRARY mode..";
         }
 
       bins.push_back(current_rounded_mz);
