@@ -259,6 +259,13 @@ TandemWrapperRun::writeFinalTandemOutput(
     {
     }
 
+  if(m_conversionTime != 0)
+    {
+      wrap_output.setInputParameters(
+        "timing, tandemwrapper conversion time (sec)",
+        QString("%1").arg(m_conversionTime / 1000));
+    }
+
   if(wrap_output.readFile(tmp_tandem_output))
     {
     }
@@ -457,6 +464,9 @@ TandemWrapperRun::convertOrginalMsData2mzXmlData(const QString &origin,
       // QFileInfo(*_p_ofile).absoluteFilePath();
       if(output_file.open(QIODevice::WriteOnly))
         {
+          QElapsedTimer timer;
+          m_conversionTime = 0;
+          timer.start();
           p_mzxml_output = new pappso::MzxmlOutput(
             *mp_monitor, QTextStream(&output_file).device());
 
@@ -469,6 +479,10 @@ TandemWrapperRun::convertOrginalMsData2mzXmlData(const QString &origin,
           p_mzxml_output->close();
 
           delete p_mzxml_output;
+          m_conversionTime = timer.elapsed();
+
+          mp_monitor->setStatus(QObject::tr("Conversion finished in %1 seconds")
+                                  .arg(m_conversionTime / 1000));
         }
       else
         {
