@@ -80,22 +80,37 @@ Utils::writeLexicalOrderedString(QTextStream *p_out, unsigned int num)
 int
 Utils::zeroDecimalsInValue(pappso_double value)
 {
+  //qDebug() << qSetRealNumberPrecision(10) << "Double value: " << value;
+
   int intPart = static_cast<int>(value);
 
+  //qDebug() << "int part:" << intPart;
+
   double decimalPart = value - intPart;
 
-  int count = -1;
+  //qDebug() << qSetRealNumberPrecision(10) << "decimal part: " << decimalPart;
 
-  while(1)
+  int count = 0;
+
+  while(decimalPart > 0)
     {
       ++count;
 
       decimalPart *= 10;
 
-      if(decimalPart > 1)
-        return count;
+      //qDebug() << "Iteration " << count << "decimal part:" << decimalPart;
+
+      if(decimalPart >= 1)
+        {
+          //qDebug() << "Because decimal part " << decimalPart
+                   //<< "is >= 1, breaking loop while count is " << count << ".";
+
+          break;
+        }
     }
 
+  //qDebug() << "Returning count:" << count;
+
   return count;
 }
 
