@@ -41,6 +41,7 @@
 #include <QMutexLocker>
 #include <QThread>
 #include <set>
+#include <QtConcurrent>
 
 namespace pappso
 {
@@ -121,7 +122,7 @@ TimsData::TimsData(QDir timsDataDirectory)
   fillFrameIdDescrList();
 
   // get number of scans
-  if(!q.exec("SELECT SUM(NumScans) FROM Frames"))
+  if(!q.exec("SELECT SUM(NumScans),COUNT(Id) FROM Frames"))
     {
       qDebug();
       throw PappsoException(
@@ -135,7 +136,8 @@ TimsData::TimsData(QDir timsDataDirectory)
     }
   if(q.next())
     {
-      m_totalNumberOfScans = q.value(0).toLongLong();
+      m_totalNumberOfScans  = q.value(0).toLongLong();
+      m_totalNumberOfFrames = q.value(1).toLongLong();
     }
 
   if(!q.exec("select * from MzCalibration;"))
@@ -198,11 +200,22 @@ TimsData::TimsData(QDir timsDataDirectory)
           .arg(q.lastError().driverText())
           .arg(q.lastError().nativeErrorCode()));
     }
+
+  m_mapFramesRecord.resize(m_totalNumberOfFrames + 1);
   while(q.next())
     {
       QSqlRecord record = q.record();
-      m_mapFramesRecord.insert(std::pair<std::size_t, QSqlRecord>(
-        record.value(8).toULongLong(), record));
+      TimsFrameRecord &frame_record =
+        m_mapFramesRecord[record.value(8).toULongLong()];
+
+      frame_record.tims_offset         = record.value(0).toULongLong();
+      frame_record.accumulation_time   = record.value(1).toDouble();
+      frame_record.mz_calibration_id   = record.value(2).toULongLong();
+      frame_record.frame_t1            = record.value(3).toDouble();
+      frame_record.frame_t2            = record.value(4).toDouble();
+      frame_record.frame_time          = record.value(5).toDouble();
+      frame_record.msms_type           = record.value(6).toInt();
+      frame_record.tims_calibration_id = record.value(7).toULongLong();
     }
 
   mcsp_ms2Filter = std::make_shared<pappso::FilterSuiteString>(
@@ -216,6 +229,12 @@ TimsData::TimsData(QDir timsDataDirectory)
   qDebug();
 }
 
+void
+TimsData::setMonoThread(bool is_mono_thread)
+{
+  m_isMonoThread = is_mono_thread;
+}
+
 QSqlDatabase
 TimsData::openDatabaseConnection() const
 {
@@ -301,6 +320,8 @@ TimsData::fillFrameIdDescrList()
   TimsFrameSPtr tims_frame;
   bool index_found = false;
   std::size_t timsId;
+  /** @brief number of scans in mobility dimension (number of TOF scans)
+   */
   std::size_t numberScans;
   std::size_t cumulScans = 0;
   while(q.next() && (!index_found))
@@ -406,8 +427,8 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId)
 
   qDebug() << " timsId=" << timsId;
 
-  auto it_map_record_frames = m_mapFramesRecord.find(timsId);
-  if(it_map_record_frames == m_mapFramesRecord.end())
+  const TimsFrameRecord &frame_record = m_mapFramesRecord[timsId];
+  if(timsId > m_totalNumberOfScans)
     {
       throw ExceptionNotFound(
         QObject::tr("ERROR Frames database id %1 not found").arg(timsId));
@@ -416,17 +437,15 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId)
 
 
   tims_frame = std::make_shared<TimsFrameBase>(
-    TimsFrameBase(timsId, it_map_record_frames->second.value(0).toULongLong()));
+    TimsFrameBase(timsId, frame_record.tims_offset));
 
-  auto it_map_record = m_mapMzCalibrationRecord.find(
-    it_map_record_frames->second.value(2).toInt());
+  auto it_map_record =
+    m_mapMzCalibrationRecord.find(frame_record.mz_calibration_id);
   if(it_map_record != m_mapMzCalibrationRecord.end())
     {
 
-      double T1_frame =
-        it_map_record_frames->second.value(3).toDouble(); // Frames.T1
-      double T2_frame =
-        it_map_record_frames->second.value(4).toDouble(); // Frames.T2
+      double T1_frame = frame_record.frame_t1; // Frames.T1
+      double T2_frame = frame_record.frame_t2; // Frames.T2
 
 
       tims_frame.get()->setMzCalibrationInterfaceSPtr(
@@ -437,18 +456,17 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId)
     {
       throw ExceptionNotFound(
         QObject::tr("ERROR MzCalibration database id %1 not found")
-          .arg(it_map_record_frames->second.value(2).toInt()));
+          .arg(frame_record.mz_calibration_id));
     }
 
-  tims_frame.get()->setAccumulationTime(
-    it_map_record_frames->second.value(1).toDouble());
+  tims_frame.get()->setAccumulationTime(frame_record.accumulation_time);
 
-  tims_frame.get()->setTime(it_map_record_frames->second.value(5).toDouble());
-  tims_frame.get()->setMsMsType(it_map_record_frames->second.value(6).toUInt());
+  tims_frame.get()->setTime(frame_record.frame_time);
+  tims_frame.get()->setMsMsType(frame_record.msms_type);
 
 
-  auto it_map_record_tims_calibration = m_mapTimsCalibrationRecord.find(
-    it_map_record_frames->second.value(7).toInt());
+  auto it_map_record_tims_calibration =
+    m_mapTimsCalibrationRecord.find(frame_record.tims_calibration_id);
   if(it_map_record_tims_calibration != m_mapTimsCalibrationRecord.end())
     {
 
@@ -469,7 +487,7 @@ TimsData::getTimsFrameBaseCstSPtr(std::size_t timsId)
     {
       throw ExceptionNotFound(
         QObject::tr("ERROR TimsCalibration database id %1 not found")
-          .arg(it_map_record_frames->second.value(7).toInt()));
+          .arg(frame_record.tims_calibration_id));
     }
 
   return tims_frame;
@@ -518,7 +536,6 @@ TimsData::getTimsFrameCstSPtr(std::size_t timsId)
   qDebug() << " timsId=" << timsId
            << " m_mapFramesRecord.size()=" << m_mapFramesRecord.size();
 
-  QMutexLocker lock(&m_mutex);
   /*
      for(auto pair_i : m_mapFramesRecord)
      {
@@ -527,31 +544,29 @@ TimsData::getTimsFrameCstSPtr(std::size_t timsId)
      }
      */
 
-  auto it_map_record_frames = m_mapFramesRecord.find(timsId);
-  if(it_map_record_frames == m_mapFramesRecord.end())
+  const TimsFrameRecord &frame_record = m_mapFramesRecord[timsId];
+  if(timsId > m_totalNumberOfScans)
     {
-
       throw ExceptionNotFound(
         QObject::tr("ERROR Frames database id %1 not found").arg(timsId));
     }
 
-
   TimsFrameSPtr tims_frame;
 
 
-  tims_frame = mpa_timsBinDec->getTimsFrameSPtrByOffset(
-    timsId, it_map_record_frames->second.value(0).toULongLong());
+  // QMutexLocker lock(&m_mutex);
+  tims_frame =
+    mpa_timsBinDec->getTimsFrameSPtrByOffset(timsId, m_mapFramesRecord);
+  // lock.unlock();
 
   qDebug();
-  auto it_map_record = m_mapMzCalibrationRecord.find(
-    it_map_record_frames->second.value(2).toInt());
+  auto it_map_record =
+    m_mapMzCalibrationRecord.find(frame_record.mz_calibration_id);
   if(it_map_record != m_mapMzCalibrationRecord.end())
     {
 
-      double T1_frame =
-        it_map_record_frames->second.value(3).toDouble(); // Frames.T1
-      double T2_frame =
-        it_map_record_frames->second.value(4).toDouble(); // Frames.T2
+      double T1_frame = frame_record.frame_t1; // Frames.T1
+      double T2_frame = frame_record.frame_t2; // Frames.T2
 
 
       tims_frame.get()->setMzCalibrationInterfaceSPtr(
@@ -562,18 +577,17 @@ TimsData::getTimsFrameCstSPtr(std::size_t timsId)
     {
       throw ExceptionNotFound(
         QObject::tr("ERROR MzCalibration database id %1 not found")
-          .arg(it_map_record_frames->second.value(2).toInt()));
+          .arg(frame_record.mz_calibration_id));
     }
 
-  tims_frame.get()->setAccumulationTime(
-    it_map_record_frames->second.value(1).toDouble());
+  tims_frame.get()->setAccumulationTime(frame_record.accumulation_time);
 
-  tims_frame.get()->setTime(it_map_record_frames->second.value(5).toDouble());
-  tims_frame.get()->setMsMsType(it_map_record_frames->second.value(6).toUInt());
+  tims_frame.get()->setTime(frame_record.frame_time);
+  tims_frame.get()->setMsMsType(frame_record.msms_type);
 
   qDebug();
-  auto it_map_record_tims_calibration = m_mapTimsCalibrationRecord.find(
-    it_map_record_frames->second.value(7).toInt());
+  auto it_map_record_tims_calibration =
+    m_mapTimsCalibrationRecord.find(frame_record.tims_calibration_id);
   if(it_map_record_tims_calibration != m_mapTimsCalibrationRecord.end())
     {
 
@@ -594,7 +608,7 @@ TimsData::getTimsFrameCstSPtr(std::size_t timsId)
     {
       throw ExceptionNotFound(
         QObject::tr("ERROR TimsCalibration database id %1 not found")
-          .arg(it_map_record_frames->second.value(7).toInt()));
+          .arg(frame_record.tims_calibration_id));
     }
   qDebug();
   return tims_frame;
@@ -1037,6 +1051,7 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(
               mass_spectrum.setEmptyMassSpectrum(true);
             }
         }
+      qDebug();
     }
 
   catch(PappsoException &error)
@@ -1053,12 +1068,14 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(
 TimsFrameBaseCstSPtr
 TimsData::getTimsFrameBaseCstSPtrCached(std::size_t timsId)
 {
-  // QMutexLocker locker(&m_mutex);
+  QMutexLocker locker(&m_mutex);
   for(auto &tims_frame : m_timsFrameBaseCache)
     {
       if(tims_frame.get()->getId() == timsId)
         {
           m_timsFrameBaseCache.push_back(tims_frame);
+          if(m_timsFrameBaseCache.size() > m_cacheSize)
+            m_timsFrameBaseCache.pop_front();
           return tims_frame;
         }
     }
@@ -1072,7 +1089,66 @@ TimsData::getTimsFrameBaseCstSPtrCached(std::size_t timsId)
 TimsFrameCstSPtr
 TimsData::getTimsFrameCstSPtrCached(std::size_t timsId)
 {
-  // QMutexLocker locker(&m_mutex);
+  qDebug();
+  QMutexLocker locker(&m_mutex);
+  for(auto &tims_frame : m_timsFrameCache)
+    {
+      if(tims_frame.get()->getId() == timsId)
+        {
+          m_timsFrameCache.push_back(tims_frame);
+          if(m_timsFrameCache.size() > m_cacheSize)
+            m_timsFrameCache.pop_front();
+          return tims_frame;
+        }
+    }
+  pappso::TimsFrameCstSPtr frame_sptr = getTimsFrameCstSPtr(timsId);
+
+  // locker.relock();
+  qDebug();
+
+  m_timsFrameCache.push_back(frame_sptr);
+  if(m_timsFrameCache.size() > m_cacheSize)
+    m_timsFrameCache.pop_front();
+  qDebug();
+  return m_timsFrameCache.back();
+
+
+  /*
+// the frame is not in the cache
+if(std::find(m_someoneIsLoadingFrameId.begin(),
+             m_someoneIsLoadingFrameId.end(),
+             timsId) == m_someoneIsLoadingFrameId.end())
+  {
+    // not found, we are alone on this frame
+    m_someoneIsLoadingFrameId.push_back(timsId);
+    qDebug();
+    //locker.unlock();
+    pappso::TimsFrameCstSPtr frame_sptr = getTimsFrameCstSPtr(timsId);
+
+   // locker.relock();
+    qDebug();
+    m_someoneIsLoadingFrameId.erase(
+      std::find(m_someoneIsLoadingFrameId.begin(),
+                m_someoneIsLoadingFrameId.end(),
+                timsId));
+
+    m_timsFrameCache.push_back(frame_sptr);
+    if(m_timsFrameCache.size() > m_cacheSize)
+      m_timsFrameCache.pop_front();
+    qDebug();
+    return m_timsFrameCache.back();
+  }
+else
+  {
+    // this frame is loading by someone else, we have to wait
+    qDebug();
+    // locker.unlock();
+    // std::size_t another_frame_id = timsId;
+    while(true)
+      {
+        QThread::usleep(1);
+  // locker.relock();
+
   for(auto &tims_frame : m_timsFrameCache)
     {
       if(tims_frame.get()->getId() == timsId)
@@ -1081,11 +1157,10 @@ TimsData::getTimsFrameCstSPtrCached(std::size_t timsId)
           return tims_frame;
         }
     }
-
-  m_timsFrameCache.push_back(getTimsFrameCstSPtr(timsId));
-  if(m_timsFrameCache.size() > m_cacheSize)
-    m_timsFrameCache.pop_front();
-  return m_timsFrameCache.back();
+  // locker.unlock();
+}
+} // namespace pappso
+*/
 }
 
 void
@@ -1111,6 +1186,7 @@ pappso::TimsData::getXicCoordTimsFromPrecursorId(std::size_t precursor_id,
     {
       if(m_mapXicCoordRecord.size() == 0)
         {
+          QMutexLocker lock(&m_mutex);
           // Go get records!
 
           // We proceed in this way:
@@ -1341,6 +1417,7 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
 {
   try
     {
+      qDebug();
       MassSpectrumId spectrum_id;
 
       spectrum_id.setSpectrumIndex(spectrum_descr.ms2_index);
@@ -1396,7 +1473,9 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
       for(std::size_t tims_id : spectrum_descr.tims_frame_list)
         {
           qDebug() << " precursor_index=" << spectrum_descr.precursor_id
-                   << " tims_id=" << tims_id;
+                   << " tims_id=" << tims_id
+                   << (std::size_t)QThread::currentThreadId();
+          ;
           if(want_binary_data)
             {
               qDebug() << "bindec";
@@ -1406,6 +1485,8 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
             {
               tims_frame = getTimsFrameBaseCstSPtrCached(tims_id);
             }
+          qDebug() << (std::size_t)QThread::currentThreadId();
+          ;
           if(first)
             {
               mass_spectrum.setRtInSeconds(tims_frame.get()->getTime());
@@ -1448,11 +1529,14 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
                           .arg(tims_frame.get()->getId()));
                     }
                 }
+              qDebug() << (std::size_t)QThread::currentThreadId();
+              ;
               tims_frame.get()->cumulateScansInRawMap(
                 raw_spectrum,
                 spectrum_descr.scan_mobility_start,
                 spectrum_descr.scan_mobility_end);
-              qDebug();
+              qDebug() << (std::size_t)QThread::currentThreadId();
+              ;
             }
           previous_frame = tims_frame;
         }
@@ -1603,23 +1687,12 @@ pappso::TimsData::ms2ReaderSpectrumCollectionByMsLevel(
 
   qDebug() << "qprecursor_list.at()=" << qprecursor_list.at();
   qprecursor_list.first();
+  std::vector<pappso::TimsData::SpectrumDescr> spectrum_description_list;
+  spectrum_descr.precursor_id = 0;
   // std::size_t i = 0;
+
   do
     {
-      qDebug() << "i=" << i;
-      // If the user of this reader instance wants to stop reading the
-      // spectra, then break this loop.
-      if(handler.shouldStop())
-        {
-          qDebug() << "The operation was cancelled. Breaking the loop.";
-          throw ExceptionInterrupted(
-            QObject::tr("reading TimsTOF job cancelled by the user :\n%1")
-              .arg(m_timsDataDirectory.absoluteFilePath("analysis.tdf")));
-        }
-      qDebug() << " spectrum_descr.precursor_id ="
-               << spectrum_descr.precursor_id;
-      qDebug() << " qprecursor_list.value(6).toLongLong() ="
-               << qprecursor_list.value(6).toLongLong();
 
       if(spectrum_descr.precursor_id !=
          (std::size_t)qprecursor_list.value(6).toLongLong())
@@ -1627,9 +1700,7 @@ pappso::TimsData::ms2ReaderSpectrumCollectionByMsLevel(
           // new precursor
           if(spectrum_descr.precursor_id > 0)
             {
-              qDebug();
-              ms2ReaderGenerateMS1MS2Spectrum(
-                msrun_id, handler, spectrum_descr, ms_level);
+              spectrum_description_list.push_back(spectrum_descr);
             }
 
           spectrum_descr.tims_frame_list.clear();
@@ -1679,19 +1750,99 @@ pappso::TimsData::ms2ReaderSpectrumCollectionByMsLevel(
       i++;
     }
   while(qprecursor_list.next());
-  qDebug();
+
+  // last One
+
+  // new precursor
   if(spectrum_descr.precursor_id > 0)
     {
-      qDebug();
-      ms2ReaderGenerateMS1MS2Spectrum(
-        msrun_id, handler, spectrum_descr, ms_level);
+      spectrum_description_list.push_back(spectrum_descr);
     }
+
+
+  QString local_filepath = m_timsDataDirectory.absoluteFilePath("analysis.tdf");
+
+  if(m_isMonoThread)
+    {
+      for(SpectrumDescr &spectrum_descr : spectrum_description_list)
+        {
+
+          std::vector<QualifiedMassSpectrum> mass_spectrum_list;
+          ms2ReaderGenerateMS1MS2Spectrum(
+            msrun_id, mass_spectrum_list, handler, spectrum_descr, ms_level);
+
+          for(auto &qualified_spectrum : mass_spectrum_list)
+            {
+              handler.setQualifiedMassSpectrum(qualified_spectrum);
+            }
+
+          if(handler.shouldStop())
+            {
+              qDebug() << "The operation was cancelled. Breaking the loop.";
+              throw ExceptionInterrupted(
+                QObject::tr("reading TimsTOF job cancelled by the user :\n%1")
+                  .arg(local_filepath));
+            }
+        }
+    }
+  else
+    {
+
+
+      TimsData *itself                                            = this;
+      pappso::SpectrumCollectionHandlerInterface *pointer_handler = &handler;
+
+
+      std::function<std::vector<QualifiedMassSpectrum>(
+        pappso::TimsData::SpectrumDescr &)>
+        generate_spectrum = [itself, msrun_id, pointer_handler, ms_level](
+                              pappso::TimsData::SpectrumDescr &spectrum_descr)
+        -> std::vector<QualifiedMassSpectrum> {
+        std::vector<QualifiedMassSpectrum> mass_spectrum_list;
+        itself->ms2ReaderGenerateMS1MS2Spectrum(msrun_id,
+                                                mass_spectrum_list,
+                                                *pointer_handler,
+                                                spectrum_descr,
+                                                ms_level);
+
+
+        return mass_spectrum_list;
+      };
+
+      QFuture<std::size_t> res;
+      res = QtConcurrent::mappedReduced<std::size_t>(
+        spectrum_description_list.begin(),
+        spectrum_description_list.end(),
+        generate_spectrum,
+        [pointer_handler, res, local_filepath](
+          std::size_t &result,
+          std::vector<QualifiedMassSpectrum> qualified_spectrum_list) {
+          for(auto &qualified_spectrum : qualified_spectrum_list)
+            {
+              pointer_handler->setQualifiedMassSpectrum(qualified_spectrum);
+            }
+
+          if(pointer_handler->shouldStop())
+            {
+              qDebug() << "The operation was cancelled. Breaking the loop.";
+              throw ExceptionInterrupted(
+                QObject::tr("reading TimsTOF job cancelled by the user :\n%1")
+                  .arg(local_filepath));
+            }
+          result++;
+        },
+        QtConcurrent::OrderedReduce);
+      res.waitForFinished();
+    }
+  handler.loadingEnded();
+  mpa_timsBinDec->closeLinearRead();
 }
 
 
 void
 pappso::TimsData::ms2ReaderGenerateMS1MS2Spectrum(
   const MsRunIdCstSPtr &msrun_id,
+  std::vector<QualifiedMassSpectrum> &qualified_mass_spectrum_list,
   pappso::SpectrumCollectionHandlerInterface &handler,
   pappso::TimsData::SpectrumDescr &spectrum_descr,
   unsigned int ms_level)
@@ -1702,27 +1853,23 @@ pappso::TimsData::ms2ReaderGenerateMS1MS2Spectrum(
   // current run via the mass spectrum id member datum.
   if((ms_level == 0) || (ms_level == 1))
     {
-
-      QualifiedMassSpectrum mass_spectrum_ms1;
-      getQualifiedMs1MassSpectrumByPrecursorId(msrun_id,
-                                               mass_spectrum_ms1,
-                                               spectrum_descr,
-                                               handler.needMsLevelPeakList(1));
-
-      handler.setQualifiedMassSpectrum(mass_spectrum_ms1);
+      qualified_mass_spectrum_list.push_back(QualifiedMassSpectrum());
+      getQualifiedMs1MassSpectrumByPrecursorId(
+        msrun_id,
+        qualified_mass_spectrum_list.back(),
+        spectrum_descr,
+        handler.needMsLevelPeakList(1));
     }
   if((ms_level == 0) || (ms_level == 2))
     {
-
-
-      QualifiedMassSpectrum mass_spectrum_ms2;
-      getQualifiedMs2MassSpectrumByPrecursorId(msrun_id,
-                                               mass_spectrum_ms2,
-                                               spectrum_descr,
-                                               handler.needMsLevelPeakList(2));
-
-      handler.setQualifiedMassSpectrum(mass_spectrum_ms2);
+      qualified_mass_spectrum_list.push_back(QualifiedMassSpectrum());
+      getQualifiedMs2MassSpectrumByPrecursorId(
+        msrun_id,
+        qualified_mass_spectrum_list.back(),
+        spectrum_descr,
+        handler.needMsLevelPeakList(2));
     }
+  qDebug();
 }
 
 
@@ -1813,15 +1960,13 @@ std::vector<double>
 pappso::TimsData::getRetentionTimeLine() const
 {
   std::vector<double> timeline;
-  auto it_map_record_frame     = m_mapFramesRecord.begin();
-  auto it_map_record_frame_end = m_mapFramesRecord.end();
-  while(it_map_record_frame != it_map_record_frame_end)
+  timeline.reserve(m_mapFramesRecord.size());
+  for(const TimsFrameRecord &frame_record : m_mapFramesRecord)
     {
-      // Frames.Time 5, Frames.MsMsType 6
-      if(it_map_record_frame->second.value(6).toInt() ==
-         0) // m_msMsType == 0 is MS level 1
-        timeline.push_back(it_map_record_frame->second.value(5).toDouble());
-      it_map_record_frame++;
+      if(frame_record.mz_calibration_id != 0)
+        {
+          timeline.push_back(frame_record.frame_time);
+        }
     }
   return timeline;
 }
