@@ -383,11 +383,11 @@ TimsFrameBase::getTraceFromCumulatedScansBuiltinCentroid(
               data_point_cumul.y *= 100.0 / m_accumulationTime;
 
 
-              qDebug() << "raw data x=" << data_point_cumul.x;
+              //qDebug() << "raw data x=" << data_point_cumul.x;
               // mz calibration
               data_point_cumul.x =
                 mz_calibration_p->getMzFromTofIndex(data_point_cumul.x);
-              qDebug() << "mz=" << data_point_cumul.x;
+              //qDebug() << "mz=" << data_point_cumul.x;
               local_trace.push_back(data_point_cumul);
             }
 
