@@ -62,7 +62,7 @@ class TimsFrameBase
   /**
    * Destructor
    */
-  ~TimsFrameBase();
+  virtual ~TimsFrameBase();
 
   /** @brief tells if 2 tims frame has the same calibration data
    * Usefull to know if raw data can be handled between frames
