@@ -507,11 +507,13 @@ BaseTracePlotWidget::axisDoubleClickHandler(
   [[maybe_unused]] QCPAxis::SelectablePart part,
   QMouseEvent *event)
 {
+  //qDebug();
 
   m_context.m_keyboardModifiers = QGuiApplication::queryKeyboardModifiers();
 
   if(m_context.m_keyboardModifiers & Qt::ControlModifier)
     {
+      //qDebug();
 
       // If the Ctrl modifiers is active, then both axes are to be reset. Also
       // the histories are reset also.
@@ -521,6 +523,8 @@ BaseTracePlotWidget::axisDoubleClickHandler(
     }
   else
     {
+      //qDebug();
+
       // Only the axis passed as parameter is to be rescaled.
       // Reset the range of that axis to the max view possible, but for the y
       // axis check if the Shift keyboard key is pressed. If so the full scale
@@ -690,6 +694,7 @@ BaseTracePlotWidget::axisRescale()
 void
 BaseTracePlotWidget::axisReframe()
 {
+  //qDebug();
 
   // double sorted_start_drag_point_x =
   // std::min(m_context.m_startDragPoint.x(), m_context.m_currentDragPoint.x());
@@ -772,6 +777,14 @@ BaseTracePlotWidget::axisZoom()
 void
 BaseTracePlotWidget::axisPan()
 {
+  //qDebug();
+
+  // Sanity check
+  if(!m_context.m_wasClickOnXAxis && !m_context.m_wasClickOnYAxis)
+    qFatal(
+      "This function can only be called if the mouse click was on one of the "
+      "axes");
+
   if(m_context.m_wasClickOnXAxis)
     {
       xAxis->setRange(m_context.m_xRange.lower - m_context.m_xDelta,
@@ -799,6 +812,8 @@ BaseTracePlotWidget::axisPan()
 
   updateContextXandYAxisRanges();
 
+  //qDebug() << "The updated context:" << m_context.toString();
+
   // We cannot store the new ranges in the history, because the pan operation
   // involved a huge quantity of micro-movements elicited upon each mouse move
   // cursor event so we would have a huge history.
